]> git.sesse.net Git - nageru/blobdiff - mixer.h
Add the beginnings of a very simple VU meter, based on libebur128.
[nageru] / mixer.h
diff --git a/mixer.h b/mixer.h
index de95eef1765cbb1f07feae2a4a5ce2693ee6e3e6..5b82b4e962fe0191d1b0c8b443e71aea4b01c8b0 100644 (file)
--- a/mixer.h
+++ b/mixer.h
@@ -7,14 +7,18 @@
 #undef Success
 #include <movit/effect_chain.h>
 #include <movit/flat_input.h>
+#include <ebur128.h>
 #include <functional>
 
-#include "bmusb.h"
+#include "bmusb/bmusb.h"
 #include "h264encode.h"
 #include "pbo_frame_allocator.h"
 #include "ref_counted_frame.h"
 #include "ref_counted_gl_sync.h"
 #include "theme.h"
+#include "resampler.h"
+#include "timebase.h"
+#include "httpd.h"
 
 #define NUM_CARDS 2
 
@@ -33,12 +37,15 @@ public:
        void quit();
 
        void transition_clicked(int transition_num);
+       void channel_clicked(int preview_num);
 
        enum Output {
                OUTPUT_LIVE = 0,
                OUTPUT_PREVIEW,
                OUTPUT_INPUT0,
                OUTPUT_INPUT1,
+               OUTPUT_INPUT2,
+               OUTPUT_INPUT3,
                NUM_OUTPUTS
        };
 
@@ -74,6 +81,17 @@ public:
                output_channel[output].set_frame_ready_callback(callback);
        }
 
+       typedef std::function<void(float)> audio_level_callback_t;
+       void set_audio_level_callback(audio_level_callback_t callback)
+       {
+               audio_level_callback = callback;
+       }
+
+       std::vector<std::string> get_transition_names()
+       {
+               return theme->get_transition_names(pts());
+       }
+
 private:
        void bm_frame(int card_index, uint16_t timecode,
                FrameAllocator::Frame video_frame, size_t video_offset, uint16_t video_format,
@@ -82,6 +100,9 @@ private:
        void thread_func();
        void subsample_chroma(GLuint src_tex, GLuint dst_dst);
        void release_display_frame(DisplayFrame *frame);
+       double pts() { return double(pts_int) / TIMEBASE; }
+
+       HTTPD httpd;
 
        QSurface *mixer_surface, *h264_encoder_surface;
        std::unique_ptr<movit::ResourcePool> resource_pool;
@@ -93,22 +114,28 @@ private:
        // Effects part of <display_chain>. Owned by <display_chain>.
        movit::FlatInput *display_input;
 
-       int frame = 0;
+       int64_t pts_int = 0;  // In TIMEBASE units.
 
        std::mutex bmusb_mutex;
        struct CaptureCard {
                BMUSBCapture *usb;
                std::unique_ptr<PBOFrameAllocator> frame_allocator;
 
-               // Threading stuff
-               bool thread_initialized = false;
+               // Stuff for the OpenGL context (for texture uploading).
                QSurface *surface;
                QOpenGLContext *context;
 
-               bool new_data_ready = false;  // Whether new_frame contains anything.
+               bool new_data_ready = false;  // Whether new_frame and new_frame_audio contains anything.
+               bool should_quit = false;
                RefCountedFrame new_frame;
                GLsync new_data_ready_fence;  // Whether new_frame is ready for rendering.
+               std::vector<float> new_frame_audio;
                std::condition_variable new_data_ready_changed;  // Set whenever new_data_ready is changed.
+               unsigned dropped_frames = 0;  // Before new_frame.
+
+               std::mutex audio_mutex;
+               std::unique_ptr<Resampler> resampler;  // Under audio_mutex.
+               int last_timecode = -1;  // Unwrapped.
        };
        CaptureCard cards[NUM_CARDS];  // protected by <bmusb_mutex>
 
@@ -135,6 +162,9 @@ private:
 
        std::thread mixer_thread;
        bool should_quit = false;
+
+       audio_level_callback_t audio_level_callback = nullptr;
+       ebur128_state *r128_state = nullptr;
 };
 
 extern Mixer *global_mixer;